@@ -3,10 +3,11 @@ import logging
 import struct
 import sys
 import time
-from typing import Callable, List, Optional
+from collections import deque
+from typing import Callable, List, NamedTuple, Optional
 
 import bleak
-from bleak.exc import BleakDeviceNotFoundError
+from bleak.exc import BleakError
 
 SENSOR_DISTANCE_NOTIFICATION_UUID = "1FE7FAF9-CE63-4236-0004-000000000002"
 CLOSE_PASS_NOTIFICATION_UUID = "1FE7FAF9-CE63-4236-0004-000000000003"
@@ -21,11 +22,17 @@ logger = logging.getLogger(__name__)
 timeout_seconds: float = 15
 
 
+class DistanceMeasurement(NamedTuple):
+    sensortime: int
+    distance_l: int
+    distance_r: int
+
+
 class ObsScanner:
     unittesting: bool = False
 
     def __init__(self) -> None:
-        if sys.version_info < (3, 10):
+        if sys.version_info < (3, 10):  # pragma: no cover
             loop = asyncio.new_event_loop()
             asyncio.set_event_loop(loop)
 
@@ -63,25 +70,45 @@ class ObsBT:
         self.bt_connected: bool = False
         self.obs_address: str | None = None
         self.my_scanner: ObsScanner = scanner if scanner is not None else ObsScanner()
-        self.find_obs()
-        self.handlebar_left: str | None = None
-        self.handlebar_right: str | None = None
+        self.handlebar_left: int = 0
+        self.handlebar_right: int = 0
         self.track_id: str | None = None
         self.overtaking_callbacks: List[Callable] = []
         self.recording_callbacks: List[Callable] = []
         self.bt_connecting: bool = False
         self.unittesting: bool = False
+        self.history: deque = deque(maxlen=100)
 
-    def find_obs(self) -> None:
-        asyncio.run(self.my_scanner.run())
-        self.obs_address = self.my_scanner.obs_address
+    async def find_obs(self) -> None:
+        await self.my_scanner.run()
+        if self.obs_address is None:
+            self.obs_address = self.my_scanner.obs_address
         assert self.obs_address is not None
 
     def overtaking_handler(self, sender: str, data: bytearray) -> None:
         """Simple notification handler which prints the data received."""
         t, l, r = struct.unpack("Ihh", data)
-        logger.info(f"sensortime: {t}, Left distance {l}, right distance {r}")
+        logger.info(
+            f"Overtaking event: sensortime: {t}, Left distance {l}, right distance {r}"
+        )
         for callback in self.overtaking_callbacks:
+            callback(
+                distance_l=l - self.handlebar_left,
+                distance_r=r - self.handlebar_right,
+                handlebars_l=self.handlebar_left,
+                handlebar_r=self.handlebar_right,
+                track_id=self.track_id,
+                sensortime=t,
+                history=list(self.history),
+            )
+
+    def history_handler(self, sender: str, data: bytearray) -> None:
+        t, l, r = struct.unpack("Ihh", data)
+        self.history.append(
+            DistanceMeasurement(t, l - self.handlebar_left, r - self.handlebar_right)
+        )
+
+        for callback in self.recording_callbacks:
             callback(
                 distance_l=l - self.handlebar_left,
                 distance_r=r - self.handlebar_right,
@@ -103,6 +130,9 @@ class ObsBT:
             await client.start_notify(
                 CLOSE_PASS_NOTIFICATION_UUID, self.overtaking_handler
             )
+            await client.start_notify(
+                SENSOR_DISTANCE_NOTIFICATION_UUID, self.history_handler
+            )
             self.track_id = (await client.read_gatt_char(TRACK_ID_UUID)).decode("utf-8")
             handlebar = await client.read_gatt_char(HANDLEBAR_OFFSET_UUID)
             self.handlebar_left, self.handlebar_right = struct.unpack("hh", handlebar)
@@ -117,6 +147,7 @@ class ObsBT:
                 await asyncio.sleep(1)
 
     async def run(self) -> None:
+        await self.find_obs()
         while not self.bt_connected:
             try:
                 if not self.bt_connecting:
@@ -124,9 +155,9 @@ class ObsBT:
                     self.bt_connecting = True
                     assert isinstance(self.obs_address, str)
                     await self.connect()
-                else:
+                else:  # pragma: no cover
                     await asyncio.sleep(1)
-            except BleakDeviceNotFoundError:
+            except BleakError:  # pragma: no cover
                 logger.info(
                     f"lost connection, reconnecting. connecting={self.bt_connecting}"
                 )
